@@ -3,61 +3,19 @@ The calculate module contains a routine for calculating the
 property surface of a system.
 """
 
-from pycalphad.codegen.callables import build_phase_records
-from pycalphad import ConditionError
-from pycalphad.core.utils import point_sample, generate_dof
-from pycalphad.core.utils import endmember_matrix, unpack_kwarg
-from pycalphad.core.utils import broadcast_to, filter_phases, unpack_condition,\
-    unpack_components, get_state_variables, instantiate_models
-from pycalphad.core.light_dataset import LightDataset
-from pycalphad.core.cache import cacheit
-from pycalphad.core.phase_rec import PhaseRecord
-import pycalphad.variables as v
-import numpy as np
 import itertools
-import collections
-from xarray import Dataset, concat
 from collections import OrderedDict
-
-
-def _generate_fake_points(components, statevar_dict, energy_limit, output, maximum_internal_dof, broadcast):
-    """
-    Generate points for a fictitious hyperplane used as a starting point for energy minimization.
-    """
-    coordinate_dict = {'component': components}
-    largest_energy = float(energy_limit)
-    if largest_energy < 0:
-        largest_energy *= 0.01
-    else:
-        largest_energy *= 10
-    if broadcast:
-        output_columns = [str(x) for x in statevar_dict.keys()] + ['points']
-        statevar_shape = tuple(len(np.atleast_1d(x)) for x in statevar_dict.values())
-        coordinate_dict.update({str(key): value for key, value in statevar_dict.items()})
-        # The internal dof for the fake points are all NaNs
-        expanded_points = np.full(statevar_shape + (len(components), maximum_internal_dof), np.nan)
-        data_arrays = {'X': (output_columns + ['component'],
-                             broadcast_to(np.eye(len(components)), statevar_shape + (len(components), len(components)))),
-                       'Y': (output_columns + ['internal_dof'], expanded_points),
-                       'Phase': (output_columns, np.full(statevar_shape + (len(components),), '_FAKE_', dtype='S6')),
-                       output: (output_columns, np.full(statevar_shape + (len(components),), largest_energy))
-                       }
-    else:
-        output_columns = ['points']
-        statevar_shape = (len(components) * max([len(np.atleast_1d(x)) for x in statevar_dict.values()]),)
-        # The internal dof for the fake points are all NaNs
-        expanded_points = np.full(statevar_shape + (maximum_internal_dof,), np.nan)
-        data_arrays = {'X': (output_columns + ['component'],
-                             broadcast_to(np.tile(np.eye(len(components)), (statevar_shape[0] / len(components), 1)),
-                                                  statevar_shape + (len(components),))),
-                       'Y': (output_columns + ['internal_dof'], expanded_points),
-                       'Phase': (output_columns, np.full(statevar_shape, '_FAKE_', dtype='S6')),
-                       output: (output_columns, np.full(statevar_shape, largest_energy))
-                       }
-        # Add state variables as data variables if broadcast=False
-        data_arrays.update({str(key): (output_columns, np.repeat(value, len(components)))
-                            for key, value in statevar_dict.items()})
-    return Dataset(data_arrays, coords=coordinate_dict)
+import numpy as np
+from numpy import broadcast_to
+import pycalphad.variables as v
+from pycalphad import ConditionError
+from pycalphad.codegen.callables import build_phase_records
+from pycalphad.core.cache import cacheit
+from pycalphad.core.light_dataset import LightDataset
+from pycalphad.core.phase_rec import PhaseRecord
+from pycalphad.core.utils import endmember_matrix, extract_parameters, \
+    filter_phases, generate_dof, instantiate_models, point_sample, \
+    unpack_components, unpack_condition, unpack_kwarg
 
 
 @cacheit
@@ -113,12 +71,13 @@ def _sample_phase_constitution(phase_name, phase_constituents, sublattice_dof, c
     # Issues with this appear to be platform-dependent
     points = points[~np.isnan(points).any(axis=-1)]
     # Ensure that points has the correct dimensions and dtype
-    points = np.atleast_2d(np.asarray(points, dtype=np.float))
+    points = np.atleast_2d(np.asarray(points, dtype=np.float_))
     return points
 
 
 def _compute_phase_values(components, statevar_dict,
-                          points, phase_record, output, maximum_internal_dof, broadcast=True, fake_points=False,
+                          points, phase_record, output, maximum_internal_dof, broadcast=True,
+                          parameters=None, fake_points=False,
                           largest_energy=None):
     """
     Calculate output values for a particular phase.
@@ -141,6 +100,9 @@ def _compute_phase_values(components, statevar_dict,
     broadcast : bool
         If True, broadcast state variables against each other to create a grid.
         If False, assume state variables are given as equal-length lists (or single-valued).
+    parameters : OrderedDict {str -> float or sequence}, optional
+        Maps SymPy symbols to a scalar or 1-D array. The arrays must be equal length.
+        The corresponding PhaseRecord must have been initialized with the same parameters.
     fake_points : bool, optional (Default: False)
         If True, the first few points of the output surface will be fictitious
         points used to define an equilibrium hyperplane guaranteed to be above
@@ -180,9 +142,19 @@ def _compute_phase_values(components, statevar_dict,
     bc_statevars = np.ascontiguousarray([broadcast_to(x, points.shape[:-1]).reshape(-1) for x in statevars])
     pts = points.reshape(-1, points.shape[-1])
     dof = np.ascontiguousarray(np.concatenate((bc_statevars.T, pts), axis=1))
-    phase_output = np.zeros(dof.shape[0], order='C')
     phase_compositions = np.zeros((dof.shape[0], len(pure_elements)), order='F')
-    phase_record.obj_2d(phase_output, dof)
+
+    param_symbols, parameter_array = extract_parameters(parameters)
+    parameter_array_length = parameter_array.shape[0]
+    if parameter_array_length == 0:
+        # No parameters specified
+        phase_output = np.zeros(dof.shape[0], order='C')
+        phase_record.obj_2d(phase_output, dof)
+    else:
+        # Vectorized parameter arrays
+        phase_output = np.zeros((dof.shape[0], parameter_array_length), order='C')
+        phase_record.obj_parameters_2d(phase_output, dof, parameter_array)
+
     for el_idx in range(len(pure_elements)):
         phase_record.mass_obj_2d(phase_compositions[:, el_idx], dof, el_idx)
 
@@ -191,12 +163,21 @@ def _compute_phase_values(components, statevar_dict,
         phase_output = broadcast_to(phase_output, points.shape[:-1])
     if isinstance(phase_compositions, (float, int)):
         phase_compositions = broadcast_to(phase_output, points.shape[:-1] + (len(pure_elements),))
-    phase_output = np.asarray(phase_output, dtype=np.float)
-    phase_output.shape = points.shape[:-1]
-    phase_compositions = np.asarray(phase_compositions, dtype=np.float)
+    phase_output = np.asarray(phase_output, dtype=np.float_)
+    if parameter_array_length <= 1:
+        phase_output.shape = points.shape[:-1]
+    else:
+        phase_output.shape = points.shape[:-1] + (parameter_array_length,)
+    phase_compositions = np.asarray(phase_compositions, dtype=np.float_)
     phase_compositions.shape = points.shape[:-1] + (len(pure_elements),)
     if fake_points:
-        phase_output = np.concatenate((broadcast_to(largest_energy, points.shape[:-2] + (max_tieline_vertices,)), phase_output), axis=-1)
+        output_shape = points.shape[:-2] + (max_tieline_vertices,)
+        if parameter_array_length > 1:
+            output_shape = output_shape + (parameter_array_length,)
+            concat_axis = -2
+        else:
+            concat_axis = -1
+        phase_output = np.concatenate((broadcast_to(largest_energy, output_shape), phase_output), axis=concat_axis)
         phase_names = np.concatenate((broadcast_to('_FAKE_', points.shape[:-2] + (max_tieline_vertices,)),
                                       np.full(points.shape[:-1], phase_record.phase_name, dtype='U' + str(len(phase_record.phase_name)))), axis=-1)
     else:
@@ -230,15 +211,22 @@ def _compute_phase_values(components, statevar_dict,
         output_columns = [str(x) for x in statevar_dict.keys()] + ['points']
     else:
         output_columns = ['points']
+    if parameter_array_length > 1:
+        parameter_column = ['samples']
+        coordinate_dict['param_symbols'] = [str(x) for x in param_symbols]
+    else:
+        parameter_column = []
     data_arrays = {'X': (output_columns + ['component'], phase_compositions),
                    'Phase': (output_columns, phase_names),
                    'Y': (output_columns + ['internal_dof'], expanded_points),
-                   output: (['dim_'+str(i) for i in range(len(phase_output.shape) - len(output_columns))] + output_columns, phase_output)
+                   output: (['dim_'+str(i) for i in range(len(phase_output.shape) - (len(output_columns)+len(parameter_column)))] + output_columns + parameter_column, phase_output)
                    }
     if not broadcast:
         # Add state variables as data variables rather than as coordinates
         for sym, vals in zip(statevar_dict.keys(), statevars):
             data_arrays.update({sym: (output_columns, vals)})
+    if parameter_array_length > 1:
+        data_arrays['param_values'] = (['samples', 'param_symbols'], parameter_array)
     return LightDataset(data_arrays, coords=coordinate_dict)
 
 
@@ -339,13 +327,12 @@ def calculate(dbf, comps, phases, mode=None, output='GM', fake_points=False, bro
     # If we don't do this, sympy will get confused during substitution
     statevar_dict = dict((v.StateVariable(key), unpack_condition(value)) for key, value in kwargs.items() if key in statevar_strings)
     # Sort after default state variable check to fix gh-116
-    statevar_dict = collections.OrderedDict(sorted(statevar_dict.items(), key=lambda x: str(x[0])))
+    statevar_dict = OrderedDict(sorted(statevar_dict.items(), key=lambda x: str(x[0])))
     phase_records = build_phase_records(dbf, comps, active_phases, statevar_dict,
                                    models=models, parameters=parameters,
                                    output=output, callables=callables, build_gradients=False, build_hessians=False,
                                    verbose=kwargs.pop('verbose', False))
-    str_statevar_dict = collections.OrderedDict((str(key), unpack_condition(value)) \
-                                                for (key, value) in statevar_dict.items())
+    str_statevar_dict = OrderedDict((str(key), unpack_condition(value)) for (key, value) in statevar_dict.items())
     maximum_internal_dof = max(len(models[phase_name].site_fractions) for phase_name in active_phases)
     for phase_name, phase_obj in sorted(active_phases.items()):
         mod = models[phase_name]
@@ -361,7 +348,7 @@ def calculate(dbf, comps, phases, mode=None, output='GM', fake_points=False, bro
         fp = fake_points and (phase_name == sorted(active_phases.keys())[0])
         phase_ds = _compute_phase_values(nonvacant_components, str_statevar_dict,
                                          points, phase_record, output,
-                                         maximum_internal_dof, broadcast=broadcast,
+                                         maximum_internal_dof, broadcast=broadcast, parameters=parameters,
                                          largest_energy=float(largest_energy), fake_points=fp)
         all_phase_data.append(phase_ds)
 
