@@ -197,6 +197,7 @@ def _tdb_grammar(): #pylint: disable=R0914
     ref_phase_name = symbol_name = Word(alphanums+'_-:()/', min=1)
     # species name, e.g., CO2, AL, FE3+
     species_name = Word(alphanums+'+-*/_.', min=1) + Optional(Suppress('%'))
+    reference_key = Word(alphanums+':_-')('reference_key')
     # constituent arrays are colon-delimited
     # each subarray can be comma- or space-delimited
     constituent_array = Group(delimitedList(Group(OneOrMore(Optional(Suppress(',')) + species_name)), ':'))
@@ -216,7 +217,8 @@ def _tdb_grammar(): #pylint: disable=R0914
         Suppress(White()) + CharsNotIn(' !', exact=1) + SkipTo(LineEnd())
     # FUNCTION
     cmd_function = TCCommand('FUNCTION') + symbol_name + \
-        func_expr.setParseAction(_make_piecewise_ast)
+        func_expr.setParseAction(_make_piecewise_ast) + \
+        Optional(Suppress(reference_key)) + LineEnd()
     # ASSESSED_SYSTEMS
     cmd_ass_sys = TCCommand('ASSESSED_SYSTEMS') + SkipTo(LineEnd())
     # DEFINE_SYSTEM_DEFAULT
@@ -238,7 +240,8 @@ def _tdb_grammar(): #pylint: disable=R0914
     # PHASE
     cmd_phase = TCCommand('PHASE') + symbol_name + \
         Suppress(White()) + CharsNotIn(' !', min=1) + Suppress(White()) + \
-        Suppress(int_number) + Group(OneOrMore(float_number)) + LineEnd()
+        Suppress(int_number) + Group(OneOrMore(float_number)) + \
+        Suppress(SkipTo(LineEnd()))
     # CONSTITUENT
     cmd_constituent = TCCommand('CONSTITUENT') + symbol_name + \
         Suppress(White()) + Suppress(':') + constituent_array + \
@@ -249,7 +252,8 @@ def _tdb_grammar(): #pylint: disable=R0914
         Optional(Suppress('&') + Word(alphas+'/-', min=1, max=2), default=None) + \
         Suppress(',') + constituent_array + \
         Optional(Suppress(';') + int_number, default=0) + \
-        Suppress(')') + func_expr.setParseAction(_make_piecewise_ast)
+        Suppress(')') + func_expr.setParseAction(_make_piecewise_ast) + \
+        Optional(Suppress(reference_key)) + LineEnd()
     # Now combine the grammar together
     all_commands = cmd_element | \
                     cmd_species | \
